
local function displayName(current)
  return string.format(
    Altimate_ClassColors[current.class] or Altimate_ClassColors[__DEFAULT__],
    (current.level and current.level ..  ' ' or '') .. current.name
  )
end

local function eventPlayerEnterWorld(isInitialLogin, isReloadingUi)
  if isInitialLogin or isReloadingUi then
    local addon = "Altimate"
    local title = GetAddOnMetadata(addon, "Title") or "Unknown"
    local author = GetAddOnMetadata(addon, "Author") or "Unknown"

    print("Addon [" .. title .. "] by " .. author .. " loaded!")
  end
end

local function eventGuildRosterUpdate()
  updateGuildRoster()

  Altimate_roster[UnitName("player")].equippedItems = getEquippedItems()
end

local function eventPlayerEquipmentChanged(slotId, isSlotNowEmpty)
  Altimate_roster[UnitName("player")]['equippedItems']['slot' .. slotId] = getEquippedItemBySlotId(slotId)
end

local function eventHandler(self, event, ...)
  if "PLAYER_ENTERING_WORLD" == event then eventPlayerEnterWorld(...); end
  if "GUILD_ROSTER_UPDATE" == event then eventGuildRosterUpdate(...); end
  if "PLAYER_EQUIPMENT_CHANGED" == event then eventPlayerEquipmentChanged(...); end
end

local function chatAddon(...)
  -- print(...)
end

local function chatFilter(self, event, message, author, ...)
  local current = Altimate_roster[string.match(author, "^([^\-]+)")]
  if not current then return false; end
  local main = Altimate_roster[current.main]
  -- main name set but incorrectly spelled or main not in current roster
  if not main and current.main then main = {name = current.main, level = nil, class = "??"} end

  local display = displayName(current)
  if main then display = displayName(main) .. " as " .. displayName(current); end

  return
    false,
    "[|Hplayer:" .. author .. "|h" .. display .. "|h]: " .. message,
    '',
    ...
end

function eventDisplayTooltip(self)
  local tooltipItem = GameTooltip:GetItem()
  if tooltipItem == nil or tooltipItem == '' then return; end

  -- itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID or "itemString" or "itemName" or "itemLink")
  local _, targetItemLink, _, targetItemLevel, _, _, _, _, targetItemEquipLoc = GetItemInfo(tooltipItem)
  if targetItemEquipLoc == nil or targetItemEquipLoc == '' then return; end

  local inventoryId = Altimate_InventorySlots[targetItemEquipLoc]

  if (type(inventoryId) == "table") then
    -- print('ignoring multi slot item ring, trink, etc')
    return
  end

  local items = {}

  for _, character in pairs(Altimate_roster) do
    local characterItemLink = nil
    local characterItemLevel = nil
    local characterItemString = character['equippedItems']['slot' .. inventoryId]
    if characterItemString ~= nil and characterItemString ~= '' then
      _, characterItemLink, _, characterItemLevel = GetItemInfo(characterItemString)
    elseif characterItemString == '' then
      characterItemLink = 'EMPTY'
      characterItemLevel = ''
    end

    if characterItemLink then
      local difference = targetItemLevel - characterItemLevel
      if difference > 0 then
        items[character.name] = {
          characterItemLevel = characterItemLevel,
          characterItemLink = characterItemLink,
          targetItemLevel = targetItemLevel,
          targetItemLink = targetItemLink,
          difference = difference
        }
        -- print(character.name .. ' iLvl ' .. characterItemLevel .. ' ' .. characterItemLink .. ' vs iLvl ' .. targetItemLevel .. ' ' .. targetItemLink)
        local tip = string.format('%d ilvl upgrade for %s', targetItemLevel - characterItemLevel, character.name)
        GameTooltip:AddLine(tip, 0.1, 1, 0.1)
      end
    end
  end
end

function registerEvents(self)
  self:RegisterEvent("PLAYER_ENTERING_WORLD")
  self:RegisterEvent("GUILD_ROSTER_UPDATE")
  self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
  self:SetScript("OnEvent", eventHandler)

  -- GameTooltip:HookScript("OnShow", eventDisplayTooltip)

  ChatFrame_AddMessageEventFilter("CHAT_MSG_ADDON", chatAddon)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_OFFICER", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", chatFilter)
  ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", chatFilter)
end
